==> lsh-0.1.8.tar.gz: lsh-0.1.8/src/compress.c <==
/* compress.c
 *
 * packet compressor
 * 
 * $Id: compress.c,v 1.6 1999/08/22 21:24:04 nisse Exp $ */

/* lsh, an implementation of the ssh protocol
 *
 * Copyright (C) 1998 Balazs Scheidler, Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "compress.h"

#include "exception.h"
#include "ssh.h"
#include "xalloc.h"

#include "compress.c.x"

#include <assert.h>

/* GABA:
   (class
     (name packet_compressor)
     (super abstract_write_pipe)
     (vars
       (compressor object compress_instance)
       (connection object ssh_connection)))
*/

static void
do_packet_deflate(struct abstract_write *closure,
		  struct lsh_string *packet,
		  struct exception_handler *e)
{
  CAST(packet_compressor, self, closure);

  if (self->connection->send_compress)
    {
      packet = CODEC(self->connection->send_compress, packet, 1);
      assert(packet);
    }
  
  A_WRITE(self->super.next, packet, e);
}

static void
do_packet_inflate(struct abstract_write *closure,
		  struct lsh_string *packet,
		  struct exception_handler *e)
{
  CAST(packet_compressor, self, closure);

  if (self->connection->rec_compress)
    {
      packet = CODEC(self->connection->rec_compress, packet, 1);
      if (!packet)
	{
	  /* FIXME: It would be nice to pass the error message from zlib on
	   * to the exception handler. */
	  EXCEPTION_RAISE
	    (e, make_protocol_exception(SSH_DISCONNECT_COMPRESSION_ERROR,
					"Inflating compressed data failed."));
	  return;
	}
    }
  A_WRITE(self->super.next, packet, e);
}

struct abstract_write *make_packet_codec(struct abstract_write *next,
					 struct ssh_connection *connection,
					 int mode)
{
  NEW(packet_compressor, res);
	
  res->super.super.write = (mode == COMPRESS_INFLATE)
    ? do_packet_inflate
    : do_packet_deflate;
  
  res->super.next = next;
  res->connection = connection;

  return &res->super.super;
}

==> lsh-0.1.10.tar.gz: lsh-0.1.10/src/compress.c <==
/* compress.c
 *
 * packet compressor
 * 
 * $Id: compress.c,v 1.7 1999/08/30 19:30:48 nisse Exp $ */

/* lsh, an implementation of the ssh protocol
 *
 * Copyright (C) 1998 Balazs Scheidler, Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "compress.h"

#include "exception.h"
#include "ssh.h"
#include "xalloc.h"

#include "compress.c.x"

#include <assert.h>

/* GABA:
   (class
     (name packet_compressor)
     (super abstract_write_pipe)
     (vars
       (compressor object compress_instance)
       (connection object ssh_connection)))
*/

static void
do_packet_deflate(struct abstract_write *closure,
		  struct lsh_string *packet)
{
  CAST(packet_compressor, self, closure);

  if (self->connection->send_compress)
    {
      packet = CODEC(self->connection->send_compress, packet, 1);
      assert(packet);
    }
  
  A_WRITE(self->super.next, packet);
}

static void
do_packet_inflate(struct abstract_write *closure,
		  struct lsh_string *packet)
{
  CAST(packet_compressor, self, closure);

  if (self->connection->rec_compress)
    {
      packet = CODEC(self->connection->rec_compress, packet, 1);
      if (!packet)
	{
	  /* FIXME: It would be nice to pass the error message from zlib on
	   * to the exception handler. */
	  EXCEPTION_RAISE
	    (self->connection->e,
	     make_protocol_exception(SSH_DISCONNECT_COMPRESSION_ERROR,
				     "Inflating compressed data failed."));
	  return;
	}
    }
  A_WRITE(self->super.next, packet);
}

struct abstract_write *
make_packet_codec(struct abstract_write *next,
		  struct ssh_connection *connection,
		  int mode)
{
  NEW(packet_compressor, res);
	
  res->super.super.write = (mode == COMPRESS_INFLATE)
    ? do_packet_inflate
    : do_packet_deflate;
  
  res->super.next = next;
  res->connection = connection;

  return &res->super.super;
}
